<?php
/**
*
* phpBB MOD Challenge main script
*
* @author Martin - http://vegan-forum.de/
*
* @package challenge
* @version 0.7
* @copyright (c) 2014 Martin
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);
include($phpbb_root_path . 'includes/message_parser.' . $phpEx);

/* *** MOD settings ******************************************************** */
// the database tables of this MOD
define('MOD_CHALLENGE_CHALLENGE_TABLE',		$table_prefix . 'mod_challenge_challenge');
define('MOD_CHALLENGE_LOG_TABLE',			$table_prefix . 'mod_challenge_log');
define('MOD_CHALLENGE_MILESTONE_TABLE',		$table_prefix . 'mod_challenge_milestone');
define('MOD_CHALLENGE_PARTICIPANT_TABLE',	$table_prefix . 'mod_challenge_participant');

// how many top/latest contributors to show
define('MOD_CHALLENGE_NUM_TOP_CONTRIBUTORS',	10);
define('MOD_CHALLENGE_NUM_LATEST_CONTRIBUTORS',	10);

// is a member allowed to edit own challenges?
define('MOD_CHALLENGE_PERM_U_E_C',	true);
// is a member allowed to delete own challenges?
define('MOD_CHALLENGE_PERM_U_D_C',	false);
// is a moderator allowed to edit all challenges?
define('MOD_CHALLENGE_PERM_M_E_C',	true);
// is a moderator allowed to delete all challenges?
define('MOD_CHALLENGE_PERM_M_D_C',	true);
// is an administrator allowed to edit all challenges?
define('MOD_CHALLENGE_PERM_A_E_C',	true);
// is an administrator allowed to delete all challenges?
define('MOD_CHALLENGE_PERM_A_D_C',	true);

// is a member allowed to edit own logs?
define('MOD_CHALLENGE_PERM_U_E_U_L',	true);
// is a member allowed to delete own logs?
define('MOD_CHALLENGE_PERM_U_D_U_L',	true);
// is a member allowed to edit logs of other members on own challenges?
define('MOD_CHALLENGE_PERM_U_E_O_L',	false);
// is a member allowed to delete logs of other members on own challenges?
define('MOD_CHALLENGE_PERM_U_D_O_L',	false);
// is a moderator allowed to edit all logs?
define('MOD_CHALLENGE_PERM_M_E_L',		true);
// is a moderator allowed to delete all logs?
define('MOD_CHALLENGE_PERM_M_D_L',		true);
// is an administrator allowed to edit all logs?
define('MOD_CHALLENGE_PERM_A_E_L',		true);
// is an administrator allowed to delete all logs?
define('MOD_CHALLENGE_PERM_A_D_L',		true);

// is a member allowed to create, edit and delete milestones on own challenges?
define('MOD_CHALLENGE_PERM_U_M',	true);
// is a moderator allowed to create, edit and delete milestones on all challenges?
define('MOD_CHALLENGE_PERM_M_M',	true);
// is an administrator allowed to create, edit and delete milestones on all challenges?
define('MOD_CHALLENGE_PERM_A_M',	true);
/* *** END MOD settings **************************************************** */

// start session management
$user->session_begin();
$auth->acl($user->data);
// specify l10n
$user->setup('mods/challenge');

// redirect bots to index page
if ($user->data['is_bot'])
{
	redirect(append_sid("{$phpbb_root_path}index.$phpEx"));
}

// users need to log in
if ($user->data['user_id'] == ANONYMOUS)
{
	login_box('', $user->lang['LOGIN']);
}

// grab data
$mode			= request_var('mode', '');
$challenge_id	= request_var('c', 0);
$log_id			= request_var('l', 0);
$milestone_id	= request_var('m', 0);
// for sorting
$sort_key		= request_var('sk', 'a');
$sort_dir		= request_var('sd', 'a');
// for pagination
$start			= request_var('start', 0);

// check our mode...
if (!in_array($mode, array('', 'listchallenge', 'viewchallenge', 'createchallenge', 'createlog', 'listlog', 'listparticipant', 'editchallenge', 'deletechallenge', 'editlog', 'deletelog', 'createmilestone', 'editmilestone', 'deletemilestone')))
{
	trigger_error('NO_MODE');
}

switch ($mode)
{
	case 'listchallenge':
	default:
	// display a list of all challenges
	{
		$mode = 'listchallenge';

		// sorting
		$sort_key_sql = array('a' => 'c.title', 'b' => 'u.username', 'c' => 'c.start', 'd' => 'c.end', 'e' => 'percent');
		if (!isset($sort_key_sql[$sort_key]))
		{
			$sort_key = $default_key;
		}
		$order_by .= $sort_key_sql[$sort_key] . ' ' . (($sort_dir == 'a') ? 'ASC' : 'DESC');

		// pagination and sorting parameters for URLs
		$params = $sort_params = array();
		if (isset($_REQUEST['sk']))
		{
			$params[] = urlencode('sk') . '=' . ((is_string(request_var('sk', 'a'))) ? urlencode(request_var('sk', 'a')) : request_var('sk', 'a'));
		}
		if (isset($_REQUEST['sd']))
		{
			$params[] = urlencode('sd') . '=' . ((is_string(request_var('sd', 'a'))) ? urlencode(request_var('sd', 'a')) : request_var('sd', 'a'));
		}
		$params[] = "mode=$mode";
		$sort_params[] = "mode=$mode";

		// pagination and sorting URLs
		$pagination_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $params));
		$sort_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $sort_params));

		// get challenge count
		$sql = 'SELECT COUNT(challenge_id) AS num_challenges
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE;
		$result = $db->sql_query($sql);
		$num_challenges = $db->sql_fetchfield('num_challenges');
		$db->sql_freeresult($result);

		// any challenges?
		if ($num_challenges > 0)
		{
			// nasty subselects needed to sort on "percent" column
			$sql = 'SELECT c.*, u.username, u.user_colour,
					(
					SELECT SUM(l.quantity)
					FROM ' . MOD_CHALLENGE_LOG_TABLE . ' l
					WHERE l.challenge_id = c.challenge_id
					) AS sum_quantity,
					(
					SELECT SUM(l.quantity)
					FROM ' . MOD_CHALLENGE_LOG_TABLE . ' l
					WHERE l.challenge_id = c.challenge_id
					) * 100 / c.target AS percent
				FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . ' c
				LEFT OUTER JOIN ' . USERS_TABLE . ' u ON c.user_id = u.user_id
				ORDER BY ' . $order_by;
			$result = $db->sql_query_limit($sql, $config['topics_per_page'], $start);
			while ($row = $db->sql_fetchrow($result))
			{
				// check if the logged in user is "owner" of the challenge and is allowed to modify and delete it
				// or if the logged in user is either an administrator or a global moderator
				$editpermission = ((MOD_CHALLENGE_PERM_U_E_C && (int) $user->data['user_id'] == (int) $row['user_id'])
					|| (MOD_CHALLENGE_PERM_M_E_C && $auth->acl_get('m_'))
					|| (MOD_CHALLENGE_PERM_A_E_C && $auth->acl_get('a_'))
				);
				$deletepermission = ((MOD_CHALLENGE_PERM_U_D_C && (int) $user->data['user_id'] == (int) $row['user_id'])
					|| (MOD_CHALLENGE_PERM_M_D_C && $auth->acl_get('m_'))
					|| (MOD_CHALLENGE_PERM_A_D_C && $auth->acl_get('a_'))
				);

				// if no logs for this challenge exist, the subselect returns NULL for sum_quantity
				if ($row['sum_quantity'] == null)
				{
					$row['sum_quantity'] = 0;
				}
				// cut off the decimal point and the following zeroes for a nicer display
				$target = (intval($row['target']) == $row['target'] ? intval($row['target']) : $row['target']);
				$sum_quantity = (intval($row['sum_quantity']) == $row['sum_quantity'] ? intval($row['sum_quantity']) : $row['sum_quantity']);

				// check for owner user
				$owner = ($row['username'] != null ? get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']) : $user->lang['UNKNOWN_USER']);
				$template->assign_block_vars('challengerow', array(
					'TITLE'					=> $row['title'],
					'U_VIEW_CHALLENGE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $row['challenge_id']),
					'OWNER'					=> $owner,
					'START_DATE'			=> ($row['start'] == null ? '' : date('d.m.Y', $row['start'])),
					'END_DATE'				=> ($row['end'] == null ? '' : date('d.m.Y', $row['end'])),
					'TARGET'				=> sprintf($user->lang['TARGET_UNIT'], $target, $row['unit']),
					'CURRENT'				=> sprintf($user->lang['TARGET_UNIT'], $sum_quantity, $row['unit']),
					'CURRENT_OF_TARGET'		=> sprintf($user->lang['CURRENT_OF_TARGET'], $sum_quantity, $target, $row['unit']),
					'PERCENT'				=> sprintf($user->lang['TARGET_PERCENT'], $row['percent']),
					'PERCENT_IMG'			=> '<img style="max-width: 100%; min-height: 18px; max-height: 24px;" src="' . "{$phpbb_root_path}challenge_gfx.$phpEx?mode=simplebar&p=" . sprintf('%.2f', $row['percent']) . '" alt="visual reprenstation of progress" />',
					'S_ALLOW_EDIT'			=> $editpermission,
					'S_ALLOW_DELETE'		=> $deletepermission,
					'U_EDIT_CHALLENGE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=editchallenge&amp;c=' . $row['challenge_id']),
					'U_DELETE_CHALLENGE'	=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=deletechallenge&amp;c=' . $row['challenge_id']),
				));
			}
			$db->sql_freeresult($result);
		}

		// generate page
		$template->assign_vars(array(
			'PAGINATION'	=> generate_pagination($pagination_url, $num_challenges, $config['topics_per_page'], $start),
			'PAGE_NUMBER'	=> on_page($num_challenges, $config['topics_per_page'], $start),
			'TOTAL_CHALLENGES'		=> sprintf($user->lang['LIST_CHALLENGES'], $num_challenges),

			'U_CREATE_CHALLENGE'	=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=createchallenge'),
			'U_SORT_TITLE'			=> $sort_url . '&amp;sk=a&amp;sd=' . (($sort_key == 'a' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_OWNER'			=> $sort_url . '&amp;sk=b&amp;sd=' . (($sort_key == 'b' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_START_DATE'		=> $sort_url . '&amp;sk=c&amp;sd=' . (($sort_key == 'c' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_END_DATE'		=> $sort_url . '&amp;sk=d&amp;sd=' . (($sort_key == 'd' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_PERCENT'		=> $sort_url . '&amp;sk=e&amp;sd=' . (($sort_key == 'e' && $sort_dir == 'a') ? 'd' : 'a'),
		));

		$page_title = $user->lang['TITLE_LIST_CHALLENGE'];
		$template_html = 'challenge_list_challenge.html';
	}
	break;

	case 'viewchallenge':
	// view a single challenge
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// get some information on challenge owner
		$sql = 'SELECT username, user_colour
			FROM ' . USERS_TABLE . '
			WHERE user_id = ' . $db->sql_escape($challenge['user_id']);
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);


		if ($row)
		{
			// user found
			$owner = get_username_string('full', $challenge['user_id'], $row['username'], $row['user_colour']);
		}
		else
		{
			// user not found
			$owner = $user->lang['UNKNOWN_USER'];
		}
		$db->sql_freeresult($result);

		// get information on logs to this challenge
		$sql = 'SELECT SUM(quantity) AS sum_quantity
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		$sum_quantity = $db->sql_fetchfield('sum_quantity');
		$db->sql_freeresult($result);

		// decode bbcode in description
		$challenge['description'] = generate_text_for_display($challenge['description'], $challenge['bbcode_uid'], $challenge['bbcode_bitfield'], $challenge['bbcode_options']);

		// get information on participants
		$participant_list = array();
		// cache profile links for top 10 and latest 10 contributor querys and for allowed participants
		$usercache = array();
		$sql = 'SELECT DISTINCT user_id
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) . '
			UNION SELECT user_id
			FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$participant_list[] = $row;
		}
		$db->sql_freeresult($result);
		for ($i = 0, $end = sizeof($participant_list); $i < $end; ++$i)
		{
			$sql = 'SELECT username, user_colour
				FROM ' . USERS_TABLE . '
				WHERE user_id = ' . $db->sql_escape($participant_list[$i]['user_id']);
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			if ($row)
			{
				// user found
				$usercache[$participant_list[$i]['user_id']] = get_username_string('full', $participant_list[$i]['user_id'], $row['username'], $row['user_colour']);
			}
			else
			{
				// user not found
				$usercache[$participant_list[$i]['user_id']] = $user->lang['UNKNOWN_USER'];
			}
			$participant_list[$i] = $usercache[$participant_list[$i]['user_id']];
			$db->sql_freeresult($result);
		}

		// get information on allowed participants
		$sql = 'SELECT COUNT(user_id) AS num_participants
			FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		$num_participants = $db->sql_fetchfield('num_participants');
		$db->sql_freeresult($result);

		if ($num_participants == 0)
		{
			// everyone is allowed to participate, show list of all participants
			$logpermission = true;
			$participant_string = implode(', ', $participant_list);
		}
		else
		{
			// only selected members are allowed, show a list of their names
			$usernames = array();
			$logpermission = false;
			$sql = 'SELECT user_id
				FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				// check if the currently logged in user is in the list of allowed participants
				if (!$logpermission && $row['user_id'] == $user->data['user_id'])
				{
					$logpermission = true;
				}
				$usernames[] = $usercache[$row['user_id']];
			}
			$db->sql_freeresult($result);
			$participant_string = sprintf($user->lang['PARTICIPATE_RESTRICTED_L'], implode(', ', $usernames));
		}

		// get top 10 contributors
		$sql = 'SELECT user_id, SUM(quantity) AS sum_quantity
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
			GROUP BY user_id
			ORDER BY sum_quantity DESC';
		$result = $db->sql_query_limit($sql, MOD_CHALLENGE_NUM_TOP_CONTRIBUTORS, 0);
		while ($row = $db->sql_fetchrow($result))
		{
			// cut off the decimal point and the following zeroes for a nicer display
			$sum_quan = (intval($row['sum_quantity']) == $row['sum_quantity'] ? intval($row['sum_quantity']) : $row['sum_quantity']);
			$template->assign_block_vars('toprow', array(
				'PROFILE'			=> $usercache[$row['user_id']],
				'SUM_QUANTITY'		=> sprintf($user->lang['TARGET_UNIT'], $sum_quan, $challenge['unit']),
			));
		}
		$db->sql_freeresult($result);

		// get latest 10 contributors
		$sql = 'SELECT user_id, date, quantity
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
			ORDER BY date DESC';
		$result = $db->sql_query_limit($sql, MOD_CHALLENGE_NUM_LATEST_CONTRIBUTORS, 0);
		while ($row = $db->sql_fetchrow($result))
		{
			// cut off the decimal point and the following zeroes for a nicer display
			$quantity = (intval($row['quantity']) == $row['quantity'] ? intval($row['quantity']) : $row['quantity']);
			$template->assign_block_vars('latestrow', array(
				'PROFILE'		=> $usercache[$row['user_id']],
				'DATE'			=> date('d.m.Y', $row['date']),
				'QUANTITY'		=> sprintf($user->lang['TARGET_UNIT'], $quantity, $challenge['unit']),
			));
		}
		$db->sql_freeresult($result);

		// get information for linked topic
		$linkedtopic = '-';
		if ($challenge['topic_id'])
		{
			$sql = 'SELECT forum_id, topic_title
				FROM ' . TOPICS_TABLE . '
				WHERE topic_id = ' . $db->sql_escape($challenge['topic_id']);
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$forumid = $row['forum_id'];
			$topicid = $challenge['topic_id'];
			$linkedtopic = '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", "f=$forumid&amp;t=$topicid&amp;view=unread#unread") . '" target="_blank">' . $row['topic_title'] . '</a>';
			$db->sql_freeresult($result);
		}

		// check if the logged in user is "owner" of the challenge and is allowed to modify and delete it
		// or if the logged in user is either an administrator or a global moderator
		$editpermission = ((MOD_CHALLENGE_PERM_U_E_C && (int) $user->data['user_id'] == (int) $challenge['user_id'])
			|| (MOD_CHALLENGE_PERM_M_E_C && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_E_C && $auth->acl_get('a_'))
		);
		$deletepermission = ((MOD_CHALLENGE_PERM_U_D_C && (int) $user->data['user_id'] == (int) $challenge['user_id'])
			|| (MOD_CHALLENGE_PERM_M_D_C && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_D_C && $auth->acl_get('a_'))
		);

		// check if the logged in user is "owner" of the challenge and is allowed to create, modify and delete milestones
		// or if the logged in user is either an administrator or a global moderator
		$milestonepermission = ((MOD_CHALLENGE_PERM_U_M && (int) $user->data['user_id'] == (int) $challenge['user_id'])
			|| (MOD_CHALLENGE_PERM_M_M && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_M && $auth->acl_get('a_'))
		);

		// are there any milestones?
		$sql = 'SELECT COUNT(milestone_id) AS num_milestones
			FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		$num_milestones = $db->sql_fetchfield('num_milestones');
		$db->sql_freeresult($result);

		if ($num_milestones > 0)
		{
			$template->assign_var('S_SHOW_MILESTONES', true);

			// get summarized log quantities to check if milestones are completed
			$summarizedlogs = array();
			$summarizedquantity = 0;
			$sql = 'SELECT date, quantity
				FROM ' . MOD_CHALLENGE_LOG_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
				ORDER BY date ASC';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$summarizedquantity += (float) $row['quantity'];
				$summarizedlogs[$row['date']] = $summarizedquantity;
			}
			$db->sql_freeresult($result);

			// get information for linked milestones
			$sql = 'SELECT milestone_id, title, target
				FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
				ORDER BY target ASC';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				// if the milestone target is bigger than the challenge target
				// (possible if the challenge is edited after a milestone is created),
				// limit the milestone target to the challenge target
				$target = min($row['target'], $challenge['target']);

				// cut off the decimal point and the following zeroes for a nicer display
				$target = (intval($target) == $target ? intval($target) : $target);

				// check if the milestone is already finished
				$finished = false;
				foreach($summarizedlogs as $summdate => $summquantity) {
					if ($summquantity >= $target)
					{
						$finished = sprintf($user->lang['MILESTONE_FINISHED'], date('d.m.Y', $summdate));
						break;
					}
				}

				// if milestone is not already finished, show percentage of completion
				if ($finished === false)
				{
					$finished = sprintf($user->lang['TARGET_PERCENT'], ($sum_quantity * 100 / $target));
				}

				$template->assign_block_vars('milestonerow', array(
					'TITLE'						=> $row['title'],
					'TARGET'					=> sprintf($user->lang['TARGET_UNIT'], $target, $challenge['unit']),
					'FINISHED'					=> ' ('. $finished .')',
					'S_ALLOW_MILESTONE_EDIT'	=> $milestonepermission,
					'S_ALLOW_MILESTONE_DELETE'	=> $milestonepermission,
					'U_MILESTONE_EDIT'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=editmilestone&amp;c=' . $challenge['challenge_id'] . '&amp;m=' . $row['milestone_id']),
					'U_MILESTONE_DELETE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=deletemilestone&amp;c=' . $challenge['challenge_id'] . '&amp;m=' . $row['milestone_id']),
				));
			}
			$db->sql_freeresult($result);
		}
		else
		{
			$template->assign_var('S_SHOW_MILESTONES', false);
		}

		// cut off the decimal point and the following zeroes for a nicer display
		$target = (intval($challenge['target']) == $challenge['target'] ? intval($challenge['target']) : $challenge['target']);
		$sum_quantity = (intval($sum_quantity) == $sum_quantity ? intval($sum_quantity) : $sum_quantity);
		$template->assign_vars(array(
			'TITLE'						=> $challenge['title'],
			'OWNER'						=> $owner,
			'DESCRIPTION'				=> $challenge['description'],
			'START_DATE'				=> ($challenge['start'] == null ? '' : date('d.m.Y', $challenge['start'])),
			'END_DATE'					=> ($challenge['end'] == null ? '' : date('d.m.Y', $challenge['end'])),
			'TARGET'					=> sprintf($user->lang['TARGET_UNIT'], $target, $challenge['unit']),
			'CURRENT'					=> sprintf($user->lang['TARGET_UNIT'], $sum_quantity, $challenge['unit']),
			'PERCENT'					=> sprintf($user->lang['TARGET_PERCENT'], ($sum_quantity * 100 / $challenge['target'])),
			'PERCENT_IMG'				=> '<img style="max-width: 100%; max-height: 50px;" src="' . "{$phpbb_root_path}challenge_gfx.$phpEx?mode=progressimg&c=" . $challenge['challenge_id'] . '" alt="visual reprenstation of progress" />',
			'TOPIC'						=> $linkedtopic,
			'PARTICIPANT_LIST'			=> $participant_string,
			'TOP_CONTRIBUTORS'			=> sprintf($user->lang['TOP_CONTRIBUTORS'], MOD_CHALLENGE_NUM_TOP_CONTRIBUTORS),
			'LATEST_CONTRIBUTORS'		=> sprintf($user->lang['LATEST_CONTRIBUTORS'], MOD_CHALLENGE_NUM_LATEST_CONTRIBUTORS),
			'U_CREATE_LOG'				=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=createlog&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_ALL_LOGS'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=listlog&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_ALL_CHALLENGES'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", ''),
			'U_LIST_PARTICIPANTS'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=listparticipant&amp;c=' . $challenge['challenge_id']),
			'S_ALLOW_EDIT'				=> $editpermission,
			'S_ALLOW_DELETE'			=> $deletepermission,
			'U_EDIT_CHALLENGE'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=editchallenge&amp;c=' . $challenge['challenge_id']),
			'U_DELETE_CHALLENGE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=deletechallenge&amp;c=' . $challenge['challenge_id']),
			'S_ALLOW_MILESTONE_CREATE'	=> $milestonepermission,
			'U_CREATE_MILESTONE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=createmilestone&amp;c=' . $challenge['challenge_id']),
			'S_ALLOW_LOG_CREATE'		=> $logpermission,
		));

		$page_title = sprintf($user->lang['TITLE_VIEW_CHALLENGE'], $challenge['title']);
		$template_html = 'challenge_view_challenge.html';
	}
	break;

	case 'createchallenge':
	case 'editchallenge':
	// create a new challenge or edit an already existing challenge - since these cases
	// are very similar and use the same template, we combine them
	{
		// when editing, we need a valid challenge
		$challenge = array();
		if ($mode == 'editchallenge')
		{
			if ($challenge_id == 0)
			{
				trigger_error('NO_CHALLENGE');
			}

			$sql = 'SELECT *
				FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
			$result = $db->sql_query($sql);
			$challenge = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$challenge)
			{
				trigger_error('NO_CHALLENGE');
			}

			// check if logged in user is permitted to edit
			$editpermission = ((MOD_CHALLENGE_PERM_U_E_C && (int) $user->data['user_id'] == (int) $challenge['user_id'])
				|| (MOD_CHALLENGE_PERM_M_E_C && $auth->acl_get('m_'))
				|| (MOD_CHALLENGE_PERM_A_E_C && $auth->acl_get('a_'))
			);

			if (!$editpermission)
			{
				trigger_error('NOT_AUTHORISED');
			}
		}

		// load default phppbb l18n for posting
		$user->setup('posting');

		// get HTML, BBCode, Smilies, Images and Flash status from board config
		$bbcode_status	= ($config['allow_bbcode']) ? true : false;
		$smilies_status	= ($config['allow_smilies']) ? true : false;
		$img_status		= $bbcode_status;
		$url_status		= ($config['allow_post_links']) ? true : false;
		$flash_status	= false;
		$quote_status	= true;

		// check if form was submitted or previewed
		$submit		= (isset($_POST['post'])) ? true : false;
		$preview	= (isset($_POST['preview'])) ? true : false;

		// get submitted values
		$post_data = array();
		if ($submit || $preview)
		{
			$post_data['enable_bbcode']		= (!$bbcode_status || isset($_POST['disable_bbcode'])) ? false : true;
			$post_data['enable_smilies']	= (!$smilies_status || isset($_POST['disable_smilies'])) ? false : true;
			$post_data['enable_urls']		= (isset($_POST['disable_magic_url'])) ? 0 : 1;
			$post_data['title']				= utf8_normalize_nfc(request_var('title', '', true));
			$post_data['description']		= utf8_normalize_nfc(request_var('message', '', true));
			$post_data['start']				= utf8_normalize_nfc(request_var('start', '', true));
			$post_data['end']				= utf8_normalize_nfc(request_var('end', '', true));
			$post_data['unit']				= utf8_normalize_nfc(request_var('unit', '', true));
			// filter anything but digits and decimal points
			$post_data['target']			= preg_replace('/[^0-9,.]/', '', utf8_normalize_nfc(request_var('target', '', true)));
			$post_data['owner'] 			= (int) utf8_normalize_nfc(request_var('user', $user->data['user_id']));
			$post_data['topic']				= utf8_normalize_nfc(request_var('topic', '', true));
			$post_data['participants']		= utf8_normalize_nfc(request_var('participants', '', true));
			$post_data['username_list']		= utf8_normalize_nfc(request_var('username_list', '', true));
			$post_data['bbcode_uid']		= ''; // will be modified by generate_text_for_storage
			$post_data['bbcode_bitfield']	= ''; // dito
			$post_data['bbcode_options']	= ''; // dito

			// creates the text blob that is saved to the db later from the user input,
			// and saves the necessary parameters to bbcode_uid etc.
			// see https://wiki.phpbb.com/Tutorial.Parsing_text
			generate_text_for_storage($post_data['description'], $post_data['bbcode_uid'], $post_data['bbcode_bitfield'], $post_data['bbcode_options'], $post_data['enable_bbcode'], $post_data['enable_urls'], $post_data['enable_smilies']);

			// check form key
			if (!check_form_key('creatingchallenge'))
			{
				$error[] = $user->lang['FORM_INVALID'];
			}

			// parse title
			if (utf8_clean_string($post_data['title']) === '')
			{
				$error[] = $user->lang['EMPTY_TITLE'];
			}

			// parse start and end date
			if (utf8_clean_string($post_data['start']) === '')
			{
				$post_data['start'] = null;
			}
			else
			{
				$post_data['start'] = parse_date(utf8_clean_string($post_data['start']));
				if ($post_data['start'] == null)
				{
					$error[] = $user->lang['INVALID_START_DATE'];
				}
			}
			if (utf8_clean_string($post_data['end']) === '')
			{
				$post_data['end'] = null;
			}
			else
			{
				$post_data['end'] = parse_date(utf8_clean_string($post_data['end']));
				if ($post_data['end'] == null)
				{
					$error[] = $user->lang['INVALID_END_DATE'];
				}
			}

			// parse unit and target
			if (utf8_clean_string($post_data['unit']) === '')
			{
				$error[] = $user->lang['EMPTY_UNIT'];
			}
			if (utf8_clean_string($post_data['target']) === '')
			{
				$error[] = $user->lang['EMPTY_TARGET'];
			}
			// NB: replace comma with dot for parsing
			else if (!is_numeric(str_replace(',', '.', utf8_clean_string($post_data['target']))))
			{
				$error[] = $user->lang['INVALID_TARGET'];
			}
			else
			{
				// NB: replace comma with dot for parsing
				$post_data['target'] = floatval(str_replace(',', '.', utf8_clean_string($post_data['target'])));
			}

			// parse topic
			if (utf8_clean_string($post_data['topic']) === '')
			{
				$post_data['topic'] = null;
			}
			else
			{
				if (is_numeric(utf8_clean_string($post_data['topic'])))
				{
					$topicid = utf8_clean_string($post_data['topic']);
				}
				else
				{
					// has the user posted a complete topic link? try to parse...
					// http://.../viewtopic.php?f=2&t=1
					if (preg_match('/viewtopic.php.+t=(\d+)/', utf8_clean_string($post_data['topic']) , $matches))
					{
						$topicid = $matches[1];
					}
					else {
						// parsing failed - show error message
						$error[] = $user->lang['INVALID_TOPIC'];
						$topicid = null;
					}
				}
				if ($topicid !== null)
				{
					// get information for linked topic
					$sql = 'SELECT topic_id
						FROM ' . TOPICS_TABLE . '
						WHERE topic_id = ' . $db->sql_escape($topicid);
					$result = $db->sql_query($sql);
					$fetchedtopicid = $db->sql_fetchfield('topic_id');
					$db->sql_freeresult($result);
					if ($fetchedtopicid == $topicid)
					{
						$post_data['topic'] = $fetchedtopicid;
					}
					else
					{
						$error[] = $user->lang['INVALID_TOPIC'];
					}
				}
			}

			// parse participant setting
			$participate_userids = array();
			if (utf8_clean_string($post_data['participants']) === ''
				|| !(
					utf8_clean_string($post_data['participants']) === 'everyone'
					|| utf8_clean_string($post_data['participants']) === 'self'
					|| utf8_clean_string($post_data['participants']) === 'list'
					)
			)
			{
				$error[] = $user->lang['INVALID_PARTICIPATE'];
			}
			else if (utf8_clean_string($post_data['participants']) === 'self')
			{
				$participate_userids[] = $post_data['owner'];
			}
			else if (utf8_clean_string($post_data['participants']) === 'list')
			{
				// parse user list
				if (utf8_clean_string($post_data['username_list']) === '')
				{
					$error[] = $user->lang['PARTICIPATE_NO_USERS'];
				}
				else
				{
					// read all possible user names from input
					$possible_usernames = array();
					// NB: do not use utf8_clean_string() since it kills \n and \r
					$tok = strtok($post_data['username_list'], " ,;\n\r\t");
					while ($tok !== false) {
						$possible_usernames[] = $tok;
						$tok = strtok(" ,;\n\r\t");
					}
					$possible_usernames = array_unique($possible_usernames);

					// check if users with these names exist
					foreach ($possible_usernames as $possible_user)
					{
						$sql = 'SELECT user_id
							FROM ' . USERS_TABLE . '
							WHERE username = "' . $db->sql_escape($possible_user) .'"';
						$result = $db->sql_query($sql);
						$userid = $db->sql_fetchfield('user_id');
						$db->sql_freeresult($result);
						if ($userid == null)
						{
							$error[] = sprintf($user->lang['PARTICIPATE_UNKNOWN_USERS'], $possible_user);
						}
						else
						{
							$participate_userids[] = $userid;
						}
					}
				}
			}
			else
			{
				$post_data['username_list'] = null;
			}

			// parse description
			if (utf8_clean_string($post_data['description']) === '')
			{
				$error[] = $user->lang['EMPTY_DESCRIPTION'];
			}
		}

		// create preview
		if ($preview && !sizeof($error))
		{
			$preview_subject = $post_data['title'];
			$preview_message = generate_text_for_display($post_data['description'], $post_data['bbcode_uid'], $post_data['bbcode_bitfield'], $post_data['bbcode_options']);

			// get some information on challenge owner
			$sql = 'SELECT username, user_colour
				FROM ' . USERS_TABLE . '
				WHERE user_id = ' . $db->sql_escape(utf8_clean_string($post_data['owner']));
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$username = $row['username'];
			$usercolour = $row['user_colour'];
			$db->sql_freeresult($result);

			// get information for linked topic
			$linkedtopic = '-';
			if ($post_data['topic'])
			{
				$sql = 'SELECT forum_id, topic_title
					FROM ' . TOPICS_TABLE . '
					WHERE topic_id = ' . $db->sql_escape(utf8_clean_string($post_data['topic']));
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$forumid = $row['forum_id'];
				$topicid = $post_data['topic'];
				$linkedtopic = '<a href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx", "f=$forumid&amp;t=$topicid") . '">' . $row['topic_title'] . '</a>';
				$db->sql_freeresult($result);
			}

			// get information about allowed participants
			if ($post_data['participants'] == 'everyone')
			{
				$allowed_participants = sprintf($user->lang['PARTICIPATE_EVERYONE_L']);
			}
			else if ($post_data['participants'] == 'self' || $post_data['participants'] == 'list')
			{
				$user_links = array();
				$sql = 'SELECT user_id, username, user_colour
					FROM ' . USERS_TABLE . '
					WHERE ' . $db->sql_in_set('user_id', $participate_userids);
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$user_links[] = get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']);
				}
				$db->sql_freeresult($result);
				$allowed_participants = sprintf(sprintf($user->lang['PARTICIPATE_RESTRICTED_L'], implode(', ', $user_links)));
			}
			// cut off the decimal point and the following zeroes for a nicer display
			$target = (intval($post_data['target']) == $post_data['target'] ? intval($post_data['target']) : $post_data['target']);
			$template->assign_vars(array(
				'S_DISPLAY_PREVIEW'			=> true,
				'PREVIEW_SUBJECT'			=> $preview_subject,
				'PREVIEW_MESSAGE'			=> $preview_message,
				'PREVIEW_OWNER'				=> get_username_string('full', $post_data['owner'], $username, $usercolour),
				'PREVIEW_START'				=> ($post_data['start'] == null ? '' : date('d.m.Y', $post_data['start'])),
				'PREVIEW_END'				=> ($post_data['end'] == null ? '' : date('d.m.Y', $post_data['end'])),
				'PREVIEW_TOPIC'				=> $linkedtopic,
				'PREVIEW_TARGET'			=> sprintf($user->lang['TARGET_UNIT'], $target, $post_data['unit']),
				'PREVIEW_CURRENT'			=> sprintf($user->lang['TARGET_UNIT'], 0, $post_data['unit']),
				'PREVIEW_PERCENT'			=> sprintf($user->lang['TARGET_PERCENT'], 0),
				'PREVIEW_PARTICIPANT_LIST'	=> $allowed_participants,
			));
		}

		// store challenge
		if ($submit && !sizeof($error))
		{
			$sql_data = array(
				'title'				=> $post_data['title'],
				'description'		=> $post_data['description'],
				'start'				=> $post_data['start'],
				'end'				=> $post_data['end'],
				'unit'				=> $post_data['unit'],
				'target'			=> $post_data['target'],
				'user_id'			=> $post_data['owner'],
				'topic_id'			=> $post_data['topic'],
				'bbcode_uid'		=> $post_data['bbcode_uid'],
				'bbcode_bitfield'	=> $post_data['bbcode_bitfield'],
				'bbcode_options'	=> $post_data['bbcode_options']
			);

			if ($mode == 'createchallenge')
			{
				$sql = 'INSERT INTO ' . MOD_CHALLENGE_CHALLENGE_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_data);
				$db->sql_query($sql);
				$newid = $db->sql_nextid();
			}
			else if ($mode == 'editchallenge')
			{
				$sql = 'UPDATE ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $sql_data) . '
					WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
				$db->sql_query($sql);
				$newid = $challenge['challenge_id'];

				// if the challenge is edit from participants: list to participants: all, we
				// need to make sure there are no entries in participant table
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
					WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
				$db->sql_query($sql);
			}
			unset($sql_data);

			// store allowed participants - this is the same for modes createchallenge and editchallenge
			if ($post_data['participants'] == 'self' || $post_data['participants'] == 'list')
			{
				$sql_ary = array();
				foreach($participate_userids as $userid)
				{
					$sql_ary[] = array(
						'challenge_id'	=> $newid,
						'user_id'		=> $userid
					);
				}
				$db->sql_multi_insert(MOD_CHALLENGE_PARTICIPANT_TABLE, $sql_ary);
			}

			// show message and redirect
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $newid);
			meta_refresh(3, $redirect_url);
			$message = $user->lang['CHALLENGE_STORED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
			trigger_error($message);
		}

		// if editing and not yet submitted for preview, get challenge from db and prepare it for display
		if ($mode == 'editchallenge' && !$preview)
		{
			// $challenge is set at the beginnnig of $mode == 'editchallenge'
			$post_data['title']				= $challenge['title'];
			$post_data['description']		= $challenge['description'];
			$post_data['start']				= $challenge['start'];
			$post_data['end']				= $challenge['end'];
			$post_data['unit']				= $challenge['unit'];
			$post_data['target']			= $challenge['target'];
			$post_data['owner']				= $challenge['user_id'];
			$post_data['topic']				= $challenge['topic_id'];
			$post_data['bbcode_uid']		= $challenge['bbcode_uid'];
			$post_data['bbcode_bitfield']	= $challenge['bbcode_bitfield'];
			$post_data['bbcode_options']	= $challenge['bbcode_options'];

			// get usernames for allowed participants, if any
			$sql = 'SELECT COUNT(user_id) AS num_participants
				FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
			$result = $db->sql_query($sql);
			$num_participants = $db->sql_fetchfield('num_participants');
			$db->sql_freeresult($result);

			if ($num_participants == 0)
			{
				$post_data['participants'] = 'everyone';
				$post_data['username_list'] = null;
			}
			else
			{
				$usernames = array();
				$sql = 'SELECT u.username
					FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . ' p
					LEFT OUTER JOIN ' . USERS_TABLE . ' u ON p.user_id = u.user_id
					WHERE p.challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$usernames[] = $row['username'];
				}
				$db->sql_freeresult($result);
				$post_data['participants'] = 'list';
				$post_data['username_list'] = implode(', ', $usernames);
			}
		}

		// decode text for further editing
		decode_message($post_data['description'], $post_data['bbcode_uid']);

		// generate smiley listing
		generate_smilies('inline', 0);

		if ($mode == 'createchallenge')
		{
			$s_hidden_fields	 = '<input type="hidden" name="user" value="' . $user->data['user_id'] . '" />';
		}
		else if ($mode == 'editchallenge')
		{
			$s_hidden_fields	 = '<input type="hidden" name="user" value="' . $post_data['owner'] . '" />';
			$s_hidden_fields	.= '<input type="hidden" name="c" value="' . $challenge['challenge_id'] . '" />';
		}

		add_form_key('creatingchallenge');

		// cut off the decimal point and the following zeroes for a nicer display
		$target = (intval($post_data['target']) == $post_data['target'] ? intval($post_data['target']) : $post_data['target']);
		$template->assign_vars(array(
			'ERROR'							=> (sizeof($error)) ? implode('<br />', $error) : '',
			'TITLE'							=> $post_data['title'],
			'DESCRIPTION'					=> $post_data['description'],
			'START'							=> ($post_data['start'] == null ? '' : date('d.m.Y', $post_data['start'])),
			'END'							=> ($post_data['end'] == null ? '' : date('d.m.Y', $post_data['end'])),
			'UNIT'							=> $post_data['unit'],
			'TARGET'						=> ($target == 0 ? '' : $target),
			'TOPIC'							=> $post_data['topic'],
			'PARTICIPATE_EVERYONE_CHECKED'	=> (($post_data['participants'] == null || $post_data['participants'] == 'everyone') ? 'checked="checked" ' : ''),
			'PARTICIPATE_SELF_CHECKED'		=> ($post_data['participants'] == 'self' ? 'checked="checked" ' : ''),
			'PARTICIPATE_LIST_CHECKED'		=> ($post_data['participants'] == 'list' ? 'checked="checked" ' : ''),
			'USERNAME_LIST'					=> $post_data['username_list'],

			'BBCODE_STATUS'					=> ($bbcode_status) ? sprintf($user->lang['BBCODE_IS_ON'], '<a href="' . append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode') . '">', '</a>') : sprintf($user->lang['BBCODE_IS_OFF'], '<a href="' . append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode') . '">', '</a>'),
			'IMG_STATUS'					=> ($img_status) ? $user->lang['IMAGES_ARE_ON'] : $user->lang['IMAGES_ARE_OFF'],
			'FLASH_STATUS'					=> ($flash_status) ? $user->lang['FLASH_IS_ON'] : $user->lang['FLASH_IS_OFF'],
			'SMILIES_STATUS'				=> ($smilies_status) ? $user->lang['SMILIES_ARE_ON'] : $user->lang['SMILIES_ARE_OFF'],
			'URL_STATUS'					=> ($bbcode_status && $url_status) ? $user->lang['URL_IS_ON'] : $user->lang['URL_IS_OFF'],
			'MAX_FONT_SIZE'					=> (int) $config['max_post_font_size'],

			'U_FIND_USERNAME'				=> append_sid("{$phpbb_root_path}memberlist.$phpEx", "mode=searchuser&amp;form=postform&amp;field=username_list"),

			'S_POST_ACTION'					=> append_sid("{$phpbb_root_path}challenge.$phpEx", "mode=$mode"),
			'S_HIDDEN_FIELDS'				=> $s_hidden_fields,
			'S_BBCODE_ALLOWED'				=> ($bbcode_status) ? 1 : 0,
			'S_SMILIES_ALLOWED'				=> $smilies_status,
			'S_LINKS_ALLOWED'				=> $url_status,
			'S_BBCODE_IMG'					=> $img_status,
			'S_BBCODE_URL'					=> $url_status,
			'S_BBCODE_FLASH'				=> $flash_status,
			'S_BBCODE_QUOTE'				=> $quote_status,
		));

		// build custom bbcodes array
		display_custom_bbcodes();

		$page_title = ($mode == 'createchallenge' ? $user->lang['TITLE_CREATE_CHALLENGE'] : $user->lang['TITLE_EDIT_CHALLENGE']);
		$template_html = 'challenge_create_challenge.html';
	}
	break;

	case 'createlog':
	case 'editlog':
	// create a new log or edit an already existing log - since these cases
	// are very similar and use the same template, we combine them
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// check if the currently logged in user is allowed to create a log
		if ($mode == 'createlog')
			{
			// get information on allowed participants
			$sql = 'SELECT COUNT(user_id) AS num_participants
				FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
			$result = $db->sql_query($sql);
			$num_participants = $db->sql_fetchfield('num_participants');
			$db->sql_freeresult($result);

			// if no user_ids are in the participant table for this challenge,
			// everyone is allowed to create a log; otherwise check if the currently
			// logged in user is in this participant list
			if ($num_participants > 0)
			{
				$userids = array();
				$sql = 'SELECT user_id
					FROM ' . MOD_CHALLENGE_PARTICIPANT_TABLE . '
					WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$userids[] = $row['user_id'];
				}
				$db->sql_freeresult($result);
				if (!in_array($user->data['user_id'], $userids))
				{
					trigger_error('NOT_AUTHORISED');
				}
			}
		}

		// when editing, we need a valid log
		$log = array();
		if ($mode == 'editlog')
		{
			if ($log_id == 0)
			{
				trigger_error('NO_LOG');
			}

			$sql = 'SELECT *
				FROM ' . MOD_CHALLENGE_LOG_TABLE . '
				WHERE log_id = ' . $db->sql_escape(utf8_clean_string($log_id));
			$result = $db->sql_query($sql);
			$log = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$log)
			{
				trigger_error('NO_LOG');
			}

			// check if logged in user is permitted to edit
			$editpermission = ((MOD_CHALLENGE_PERM_U_E_U_L && (int) $user->data['user_id'] == (int) $log['user_id'])
				|| (MOD_CHALLENGE_PERM_U_E_O_L && (int) $user->data['user_id'] == (int) $challenge['user_id'])
				|| (MOD_CHALLENGE_PERM_M_E_L && $auth->acl_get('m_'))
				|| (MOD_CHALLENGE_PERM_A_E_L && $auth->acl_get('a_'))
			);

			if (!$editpermission)
			{
				trigger_error('NOT_AUTHORISED');
			}
		}

		// load default phppbb l18n for posting
		$user->setup('posting');

		// get HTML, BBCode, Smilies, Images and Flash status from board config
		$bbcode_status	= ($config['allow_bbcode']) ? true : false;
		$smilies_status	= ($config['allow_smilies']) ? true : false;
		$img_status		= $bbcode_status;
		$url_status		= ($config['allow_post_links']) ? true : false;
		$flash_status	= false;
		$quote_status	= true;

		// check if form was sumbitted or previewed
		$submit		= (isset($_POST['post'])) ? true : false;
		$preview	= (isset($_POST['preview'])) ? true : false;

		// get submitted values
		$post_data = array();
		if ($submit || $preview)
		{
			$post_data['enable_bbcode']		= (!$bbcode_status || isset($_POST['disable_bbcode'])) ? false : true;
			$post_data['enable_smilies']	= (!$smilies_status || isset($_POST['disable_smilies'])) ? false : true;
			$post_data['enable_urls']		= (isset($_POST['disable_magic_url'])) ? 0 : 1;
			$post_data['date']				= utf8_normalize_nfc(request_var('date', '', true));
			// filter anything but digits and decimal points
			$post_data['quantity']			= preg_replace('/[^0-9,.]/', '', utf8_normalize_nfc(request_var('quantity', '', true)));
			$post_data['comment']			= utf8_normalize_nfc(request_var('message', '', true));
			$post_data['user_id'] 			= (int) utf8_normalize_nfc(request_var('user', $user->data['user_id']));
			$post_data['challenge_id']		= (int) $challenge['challenge_id'];
			$post_data['bbcode_uid']		= ''; // will be modified by generate_text_for_storage
			$post_data['bbcode_bitfield']	= ''; // dito
			$post_data['bbcode_options']	= ''; // dito

			// creates the text blob that is saved to the db later from the user input,
			// and saves the necessary parameters to bbcode_uid etc.
			// see https://wiki.phpbb.com/Tutorial.Parsing_text
			generate_text_for_storage($post_data['comment'], $post_data['bbcode_uid'], $post_data['bbcode_bitfield'], $post_data['bbcode_options'], $post_data['enable_bbcode'], $post_data['enable_urls'], $post_data['enable_smilies']);

			// check form key
			if (!check_form_key('creatinglog'))
			{
				$error[] = $user->lang['FORM_INVALID'];
			}

			// parse date
			if (utf8_clean_string($post_data['date']) === '')
			{
				$error[] = $user->lang['EMPTY_DATE'];
			}
			else
			{
				$post_data['date'] = parse_date(utf8_clean_string($post_data['date']));
				if ($post_data['date'] == null)
				{
					$error[] = $user->lang['INVALID_DATE'];
				}
				else
				{
					// if challenge start and/or end date is set, check with log date
					if (($challenge['start'] != null && $post_data['date'] < $challenge['start']) ||
						($challenge['end'] != null && $post_data['date'] > $challenge['end']))
					{
						$error[] = sprintf($user->lang['DATE_OUT_OF_BOUNDS'], date('d.m.Y', $post_data['date']), date('d.m.Y', $challenge['start']), date('d.m.Y', $challenge['end']));
					}
				}
			}

			// parse quantity
			if (utf8_clean_string($post_data['quantity']) === '')
			{
				$error[] = $user->lang['EMPTY_QUANTITY'];
			}
			// NB: replace comma with dot for parsing
			else if (!is_numeric(str_replace(',', '.', utf8_clean_string($post_data['quantity']))))
			{
				$error[] = $user->lang['INVALID_QUANTITY'];
			}
			else
			{
				// NB: replace comma with dot for parsing
				$post_data['quantity'] = floatval(str_replace(',', '.', utf8_clean_string($post_data['quantity'])));
			}
		}

		// create preview
		if ($preview && !sizeof($error))
		{
			$preview_message = generate_text_for_display($post_data['comment'], $post_data['bbcode_uid'], $post_data['bbcode_bitfield'], $post_data['bbcode_options']);

			// get some information on user
			$sql = 'SELECT username, user_colour
				FROM ' . USERS_TABLE . '
				WHERE user_id = ' . $db->sql_escape(utf8_clean_string($post_data['user_id']));
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$username = $row['username'];
			$usercolour = $row['user_colour'];
			$db->sql_freeresult($result);

			// cut off the decimal point and the following zeroes for a nicer display
			$quantity = (intval($post_data['quantity']) == $post_data['quantity'] ? intval($post_data['quantity']) : $post_data['quantity']);
			$template->assign_vars(array(
				'S_DISPLAY_PREVIEW'		=> true,
				'PREVIEW_MESSAGE'		=> $preview_message,
				'PREVIEW_USER'			=> get_username_string('full', $post_data['user_id'], $row['username'], $row['user_colour']),
				'PREVIEW_DATE'			=> date('d.m.Y', $post_data['date']),
				'PREVIEW_QUANTITY'		=> sprintf($user->lang['TARGET_UNIT'], $quantity, $challenge['unit']),
			));
		}

		// store log
		if ($submit && !sizeof($error))
		{
			$sql_data = array(
				'date'				=> $post_data['date'],
				'quantity'			=> $post_data['quantity'],
				'comment'			=> $post_data['comment'],
				'user_id'			=> $post_data['user_id'],
				'challenge_id'		=> $post_data['challenge_id'],
				'bbcode_uid'		=> $post_data['bbcode_uid'],
				'bbcode_bitfield'	=> $post_data['bbcode_bitfield'],
				'bbcode_options'	=> $post_data['bbcode_options']
			);

			if ($mode == 'createlog')
			{
				$sql = 'INSERT INTO ' . MOD_CHALLENGE_LOG_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_data);
				$db->sql_query($sql);
			}
			else if ($mode == 'editlog')
			{
				$sql = 'UPDATE ' . MOD_CHALLENGE_LOG_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $sql_data) . '
					WHERE log_id = ' . $db->sql_escape($log['log_id']);
				$db->sql_query($sql);
			}
			unset($sql_data);

			// show message and redirect
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $post_data['challenge_id']);
			meta_refresh(3, $redirect_url);
			$message = $user->lang['LOG_STORED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
			trigger_error($message);
		}

		// if editing and not yet submitted for preview, get log from db and prepare it for display
		if ($mode == 'editlog' && !$preview)
		{
			// $challenge and $log are set at the beginnnig of $mode == 'editlog'
			$post_data['date']				= $log['date'];
			$post_data['quantity']			= $log['quantity'];
			$post_data['comment']			= $log['comment'];
			$post_data['user_id']			= $log['user_id'];
			$post_data['bbcode_uid']		= $log['bbcode_uid'];
			$post_data['bbcode_bitfield']	= $log['bbcode_bitfield'];
			$post_data['bbcode_options']	= $log['bbcode_options'];
		}

		// decode text for further editing
		decode_message($post_data['comment'], $post_data['bbcode_uid']);

		// generate smiley listing
		generate_smilies('inline', 0);

		$s_hidden_fields		 = '<input type="hidden" name="c" value="' . $challenge['challenge_id'] . '" />';
		if ($mode == 'createlog')
		{
			$s_hidden_fields	.= '<input type="hidden" name="user" value="' . $user->data['user_id'] . '" />';
		}
		else if ($mode == 'editlog')
		{
			$s_hidden_fields	.= '<input type="hidden" name="user" value="' . $post_data['user_id'] . '" />';
			$s_hidden_fields	.= '<input type="hidden" name="l" value="' . $log['log_id'] . '" />';
		}

		add_form_key('creatinglog');

		// cut off the decimal point and the following zeroes for a nicer display
		$quantity = (intval($post_data['quantity']) == $post_data['quantity'] ? intval($post_data['quantity']) : $post_data['quantity']);
		$template->assign_vars(array(
			'ERROR'					=> (sizeof($error)) ? implode('<br />', $error) : '',
			'DATE'					=> ($post_data['date'] == null ? date('d.m.Y') : date('d.m.Y', $post_data['date'])),
			'QUANTITY'				=> ($quantity == 0 ? '' : $quantity),
			'COMMENT'				=> $post_data['comment'],

			'BBCODE_STATUS'			=> ($bbcode_status) ? sprintf($user->lang['BBCODE_IS_ON'], '<a href="' . append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode') . '">', '</a>') : sprintf($user->lang['BBCODE_IS_OFF'], '<a href="' . append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode') . '">', '</a>'),
			'IMG_STATUS'			=> ($img_status) ? $user->lang['IMAGES_ARE_ON'] : $user->lang['IMAGES_ARE_OFF'],
			'FLASH_STATUS'			=> ($flash_status) ? $user->lang['FLASH_IS_ON'] : $user->lang['FLASH_IS_OFF'],
			'SMILIES_STATUS'		=> ($smilies_status) ? $user->lang['SMILIES_ARE_ON'] : $user->lang['SMILIES_ARE_OFF'],
			'URL_STATUS'			=> ($bbcode_status && $url_status) ? $user->lang['URL_IS_ON'] : $user->lang['URL_IS_OFF'],
			'MAX_FONT_SIZE'			=> (int) $config['max_post_font_size'],

			'S_POST_ACTION'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", "mode=$mode"),
			'S_HIDDEN_FIELDS'		=> $s_hidden_fields,
			'S_BBCODE_ALLOWED'		=> ($bbcode_status) ? 1 : 0,
			'S_SMILIES_ALLOWED'		=> $smilies_status,
			'S_LINKS_ALLOWED'		=> $url_status,
			'S_BBCODE_IMG'			=> $img_status,
			'S_BBCODE_URL'			=> $url_status,
			'S_BBCODE_FLASH'		=> $flash_status,
			'S_BBCODE_QUOTE'		=> $quote_status,
		));

		// build custom bbcodes array
		display_custom_bbcodes();

		$page_title = ($mode == 'createlog' ? $user->lang['TITLE_CREATE_LOG'] : $user->lang['TITLE_EDIT_LOG']);
		$template_html = 'challenge_create_log.html';
	}
	break;

	case 'listlog':
	// list all logs to a challenge
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// sorting
		// for list log mode we sort by "log date descending" per default!
		$sort_dir		= request_var('sd', 'd');
		$sort_key_sql = array('a' => 'l.date', 'b' => 'u.username', 'c' => 'l.quantity');
		if (!isset($sort_key_sql[$sort_key]))
		{
			$sort_key = $default_key;
		}
		$order_by .= $sort_key_sql[$sort_key] . ' ' . (($sort_dir == 'a') ? 'ASC' : 'DESC');

		// pagination and sorting parameters for URLs
		$params = $sort_params = array();
		if (isset($_REQUEST['sk']))
		{
			$params[] = urlencode('sk') . '=' . ((is_string(request_var('sk', 'a'))) ? urlencode(request_var('sk', 'a')) : request_var('sk', 'a'));
		}
		if (isset($_REQUEST['sd']))
		{
			$params[] = urlencode('sd') . '=' . ((is_string(request_var('sd', 'a'))) ? urlencode(request_var('sd', 'a')) : request_var('sd', 'a'));
		}
		$params[] = 'c=' . $challenge['challenge_id'];
		$sort_params[] = 'c=' . $challenge['challenge_id'];
		$params[] = "mode=$mode";
		$sort_params[] = "mode=$mode";

		// pagination and sorting URLs
		$pagination_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $params));
		$sort_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $sort_params));

		// get log count
		$sql = 'SELECT COUNT(log_id) AS num_logs
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		$num_logs = $db->sql_fetchfield('num_logs');
		$db->sql_freeresult($result);

		// any logs?
		if ($num_logs > 0)
		{
			$sql = 'SELECT l.*, u.username, u.user_colour
				FROM ' . MOD_CHALLENGE_LOG_TABLE . ' l
				LEFT OUTER JOIN ' . USERS_TABLE . ' u ON l.user_id = u.user_id
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) . '
				ORDER BY ' . $order_by;
			$result = $db->sql_query_limit($sql, $config['topics_per_page'], $start);
			while ($row = $db->sql_fetchrow($result))
			{
				// parse comment into display format
				$comment = generate_text_for_display($row['comment'], $row['bbcode_uid'], $row['bbcode_bitfield'], $row['bbcode_options']);

				// check if the logged in user is "owner" of the LOG and is allowed to modify and delete it
				// or if the logged in user is "owner" of the CHALLENGE and is allowed to modify and delete OTHER USERs logs
				// or if the logged in user is either an administrator or a global moderator
				$editpermission = ((MOD_CHALLENGE_PERM_U_E_U_L && (int) $user->data['user_id'] == (int) $row['user_id'])
					|| (MOD_CHALLENGE_PERM_U_E_O_L && (int) $user->data['user_id'] == (int) $challenge['user_id'])
					|| (MOD_CHALLENGE_PERM_M_E_L && $auth->acl_get('m_'))
					|| (MOD_CHALLENGE_PERM_A_E_L && $auth->acl_get('a_'))
				);
				$deletepermission = ((MOD_CHALLENGE_PERM_U_D_U_L && (int) $user->data['user_id'] == (int) $row['user_id'])
					|| (MOD_CHALLENGE_PERM_U_D_O_L && (int) $user->data['user_id'] == (int) $challenge['user_id'])
					|| (MOD_CHALLENGE_PERM_M_D_L && $auth->acl_get('m_'))
					|| (MOD_CHALLENGE_PERM_A_D_L && $auth->acl_get('a_'))
				);

				// cut off the decimal point and the following zeroes for a nicer display
				$quantity = (intval($row['quantity']) == $row['quantity'] ? intval($row['quantity']) : $row['quantity']);

				// check for user
				$loguser = ($row['username'] != null ? get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']) : $user->lang['UNKNOWN_USER']);
				$template->assign_block_vars('logrow', array(
					'DATE'				=> date('d.m.Y', $row['date']),
					'USER'				=> $loguser,
					'QUANTITY'			=> sprintf($user->lang['TARGET_UNIT'], $quantity, $challenge['unit']),
					'COMMENT'			=> $comment,
					'S_ALLOW_EDIT'		=> $editpermission,
					'S_ALLOW_DELETE'	=> $deletepermission,
					'U_EDIT_LOG'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=editlog&amp;l=' . $row['log_id'] . '&amp;c=' . $row['challenge_id']),
					'U_DELETE_LOG'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=deletelog&amp;l=' . $row['log_id']),
				));
			}
			$db->sql_freeresult($result);
		}

		// generate page
		$template->assign_vars(array(
			'PAGINATION'	=> generate_pagination($pagination_url, $num_logs, $config['topics_per_page'], $start),
			'PAGE_NUMBER'	=> on_page($num_logs, $config['topics_per_page'], $start),
			'TOTAL_LOGS'			=> sprintf($user->lang['LIST_LOGS'], $num_logs),

			'U_CREATE_LOG'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=createlog&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_CHALLENGE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_ALL_CHALLENGES'	=> append_sid("{$phpbb_root_path}challenge.$phpEx", ''),

			'U_SORT_DATE'			=> $sort_url . '&amp;sk=a&amp;sd=' . (($sort_key == 'a' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_USER'			=> $sort_url . '&amp;sk=b&amp;sd=' . (($sort_key == 'b' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_QUANTITY'		=> $sort_url . '&amp;sk=c&amp;sd=' . (($sort_key == 'c' && $sort_dir == 'a') ? 'd' : 'a'),
		));

		$page_title = $user->lang['TITLE_LIST_LOG'];
		$template_html = 'challenge_list_log.html';
	}
	break;

	case 'listparticipant':
	// list all participants to a challenge
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// sorting
		// for list participant mode we sort by "sum(quantity) descending" per default!
		$sort_dir = request_var('sd', 'd');
		$sort_key_sql = array('a' => 'sum_quantity', 'b' => 'u.username');
		if (!isset($sort_key_sql[$sort_key]))
		{
			$sort_key = $default_key;
		}
		$order_by .= $sort_key_sql[$sort_key] . ' ' . (($sort_dir == 'a') ? 'ASC' : 'DESC');

		// pagination and sorting parameters for URLs
		$params = $sort_params = array();
		if (isset($_REQUEST['sk']))
		{
			$params[] = urlencode('sk') . '=' . ((is_string(request_var('sk', 'a'))) ? urlencode(request_var('sk', 'a')) : request_var('sk', 'a'));
		}
		if (isset($_REQUEST['sd']))
		{
			$params[] = urlencode('sd') . '=' . ((is_string(request_var('sd', 'a'))) ? urlencode(request_var('sd', 'a')) : request_var('sd', 'a'));
		}
		$params[] = 'c=' . $challenge['challenge_id'];
		$sort_params[] = 'c=' . $challenge['challenge_id'];
		$params[] = "mode=$mode";
		$sort_params[] = "mode=$mode";

		// pagination and sorting URLs
		$pagination_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $params));
		$sort_url = append_sid("{$phpbb_root_path}challenge.$phpEx", implode('&amp;', $sort_params));

		// get participant count
		$num_participants = 0;
		$sql = 'SELECT user_id
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
			GROUP BY user_id';
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$num_participants++;
		}
		$db->sql_freeresult($result);

		// any participants?
		if ($num_participants > 0)
		{
			// get all participants and the sum of their logs
			$sql = 'SELECT l.user_id, u.username, u.user_colour, SUM(l.quantity) AS sum_quantity
				FROM ' . MOD_CHALLENGE_LOG_TABLE . ' l 
				LEFT OUTER JOIN ' . USERS_TABLE . ' u ON l.user_id = u.user_id
				WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
				GROUP BY l.user_id, u.username, u.user_colour
				ORDER BY ' . $order_by;
			$result = $db->sql_query_limit($sql, $config['topics_per_page'], $start);
			while ($row = $db->sql_fetchrow($result))
			{
				// cut off the decimal point and the following zeroes for a nicer display
				$sum_quan = (intval($row['sum_quantity']) == $row['sum_quantity'] ? intval($row['sum_quantity']) : $row['sum_quantity']);
				$template->assign_block_vars('participantrow', array(
					// check for user
					'PARTICIPANT'		=> ($row['username'] != null ? get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']) : $user->lang['UNKNOWN_USER']),
					'SUM_QUANTITY'		=> sprintf($user->lang['TARGET_UNIT'], $row['sum_quantity'], $challenge['unit'])
				));
			}
			$db->sql_freeresult($result);
		}

		// generate page
		$template->assign_vars(array(
			'PAGINATION'			=> generate_pagination($pagination_url, $num_participants, $config['topics_per_page'], $start),
			'PAGE_NUMBER'			=> on_page($num_participants, $config['topics_per_page'], $start),
			'TOTAL_PARTICIPANTS'	=> sprintf($user->lang['LIST_LOGS'], $num_participants),

			'U_CREATE_LOG'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=createlog&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_CHALLENGE'		=> append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $challenge['challenge_id']),
			'U_VIEW_ALL_CHALLENGES'	=> append_sid("{$phpbb_root_path}challenge.$phpEx", ''),

			'U_SORT_QUANTITY'		=> $sort_url . '&amp;sk=a&amp;sd=' . (($sort_key == 'a' && $sort_dir == 'a') ? 'd' : 'a'),
			'U_SORT_USER'			=> $sort_url . '&amp;sk=b&amp;sd=' . (($sort_key == 'b' && $sort_dir == 'a') ? 'd' : 'a'),
		));

		$page_title = $user->lang['TITLE_LIST_PARTICIPANT'];
		$template_html = 'challenge_list_participant.html';
	}
	break;

	case 'deletechallenge':
	// delete a challenge
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// check if logged in user is permitted to delete
		$deletepermission = ((MOD_CHALLENGE_PERM_U_D_C && (int) $user->data['user_id'] == (int) $challenge['user_id'])
			|| (MOD_CHALLENGE_PERM_M_D_C && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_D_C && $auth->acl_get('a_'))
		);

		if (!$deletepermission)
		{
			trigger_error('NOT_AUTHORISED');
		}
		else
		{
			if (confirm_box(true))
			{
				// this code is executed if the user confirms to delete the challenge

				// delete all logs that belong to this challenge
				$db->sql_transaction('begin');
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_LOG_TABLE . '
					WHERE ' . $db->sql_in_set('challenge_id', $challenge['challenge_id']);
				$db->sql_query($sql);

				// delete all milestones that are linked to this challenge
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
					WHERE ' . $db->sql_in_set('challenge_id', $challenge['challenge_id']);
				$db->sql_query($sql);

				// delete the challenge itself
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
					WHERE ' . $db->sql_in_set('challenge_id', $challenge['challenge_id']);
				$db->sql_query_limit($sql, 1);
				$db->sql_transaction('commit');

				$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", '');
				meta_refresh(3, $redirect_url);

				$message = $user->lang['CHALLENGE_DELETED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_ALL_CHALLENGES'] . '</a>';
				trigger_error($message);
			}
			else
			{
				// this code is executed first to show the confirmation dialog
				$s_hidden_fields = build_hidden_fields(array(
					'mode'		=> $mode,
					'c'			=> $challenge_id
				));

				confirm_box(false, 'DELETE_CHALLENGE_P', $s_hidden_fields);
			}

			// this code is executed if the user declines to delete the challenge
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", '');
			meta_refresh(3, $redirect_url);

			$message = '<a href="' . $redirect_url . '">' . $user->lang['VIEW_ALL_CHALLENGES'] . '</a>';
			trigger_error($message);
		}
	}
	break;

	case 'deletelog':
	// delete a log
	{
		if ($log_id == 0)
		{
			trigger_error('NO_LOG');
		}

		// we also need to get the "owner" of the challenge for permission check later
		$sql = 'SELECT l.*, c.user_id AS challenge_owner
			FROM ' . MOD_CHALLENGE_LOG_TABLE . ' l
			LEFT OUTER JOIN ' . MOD_CHALLENGE_CHALLENGE_TABLE . ' c ON l.challenge_id = c.challenge_id
			WHERE log_id = ' . $db->sql_escape(utf8_clean_string($log_id));
		$result = $db->sql_query($sql);
		$log = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$log)
		{
			trigger_error('NO_LOG');
		}

		// check if logged in user is permitted to delete
		$deletepermission = ((MOD_CHALLENGE_PERM_U_D_U_L && (int) $user->data['user_id'] == (int) $log['user_id'])
			|| (MOD_CHALLENGE_PERM_U_D_O_L && (int) $user->data['user_id'] == (int) $log['challenge_owner'])
			|| (MOD_CHALLENGE_PERM_M_D_L && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_D_L && $auth->acl_get('a_'))
		);

		if (!$deletepermission)
		{
			trigger_error('NOT_AUTHORISED');
		}
		else
		{
			if (confirm_box(true))
			{
				// this code is executed if the user confirms to delete the challenge

				// delete the log
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_LOG_TABLE . '
					WHERE ' . $db->sql_in_set('log_id', $log['log_id']);
				$db->sql_query_limit($sql, 1);

				$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&c=' . $challenge_id);
				meta_refresh(3, $redirect_url);

				$message = $user->lang['LOG_DELETED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
				trigger_error($message);
			}
			else
			{
				// this code is executed first to show the confirmation dialog
				$s_hidden_fields = build_hidden_fields(array(
					'mode'		=> $mode,
					'l'			=> $log_id,
					'c'			=> $log['challenge_id']
				));

				confirm_box(false, 'DELETE_LOG_P', $s_hidden_fields);
			}

			// this code is executed if the user declines to delete the log
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&c=' . $challenge_id);
			meta_refresh(3, $redirect_url);

			$message = '<a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
			trigger_error($message);
		}
	}
	break;

	case 'createmilestone':
	case 'editmilestone':
	// create a new milestone or edit an already existing milestone - since these cases
	// are very similar and use the same template, we combine them
	{
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// when editing, we need a valid milestone
		$milestone = array();
		if ($mode == 'editmilestone')
		{
			if ($milestone_id == 0)
			{
				trigger_error('NO_MILESTONE');
			}

			$sql = 'SELECT *
				FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
				WHERE milestone_id = ' . $db->sql_escape(utf8_clean_string($milestone_id));
			$result = $db->sql_query($sql);
			$milestone = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$milestone)
			{
				trigger_error('NO_MILESTONE');
			}

			// check if logged in user is permitted to edit
			$editpermission = ((MOD_CHALLENGE_PERM_U_E_C && (int) $user->data['user_id'] == (int) $challenge['user_id'])
				|| (MOD_CHALLENGE_PERM_M_E_C && $auth->acl_get('m_'))
				|| (MOD_CHALLENGE_PERM_A_E_C && $auth->acl_get('a_'))
			);

			if (!$editpermission)
			{
				trigger_error('NOT_AUTHORISED');
			}
		}

		// load default phppbb l18n for posting
		$user->setup('posting');

		// check if form was sumbitted or previewed
		$submit		= (isset($_POST['post'])) ? true : false;
		$preview	= (isset($_POST['preview'])) ? true : false;

		// get submitted values
		$post_data = array();
		if ($submit || $preview)
		{
			$post_data['title']				= utf8_normalize_nfc(request_var('title', '', true));
			// filter anything but digits and decimal points
			$post_data['target']			= preg_replace('/[^0-9,.]/', '', utf8_normalize_nfc(request_var('target', '', true)));
			$post_data['challenge_id']		= (int) $challenge['challenge_id'];

			// check form key
			if (!check_form_key('creatingmilestone'))
			{
				$error[] = $user->lang['FORM_INVALID'];
			}

			// parse title
			if (utf8_clean_string($post_data['title']) === '')
			{
				$error[] = $user->lang['EMPTY_TITLE'];
			}

			// parse target
			if (utf8_clean_string($post_data['target']) === '')
			{
				$error[] = $user->lang['EMPTY_TARGET'];
			}
			// NB: replace comma with dot for parsing
			else if (!is_numeric(str_replace(',', '.', utf8_clean_string($post_data['target']))))
			{
				$error[] = $user->lang['INVALID_TARGET'];
			}
			// check if milestone target is valid
			// NB: replace comma with dot for parsing
			else if (floatval(str_replace(',', '.', utf8_clean_string($post_data['target']))) > $challenge['target'])
			{
				$error[] = $user->lang['MILESTONE_TARGET_TOO_BIG'];
			}
			else
			{
				// NB: replace comma with dot for parsing
				$post_data['target'] = floatval(str_replace(',', '.', utf8_clean_string($post_data['target'])));
			}
		}

		// create preview
		if ($preview && !sizeof($error))
		{
			// cut off the decimal point and the following zeroes for a nicer display
			$target = (intval($post_data['target']) == $post_data['target'] ? intval($post_data['target']) : $post_data['target']);
			$template->assign_vars(array(
				'S_DISPLAY_PREVIEW'		=> true,
				'PREVIEW_TITLE'			=> $post_data['title'],
				'PREVIEW_TARGET'		=> sprintf($user->lang['TARGET_UNIT'], $target, $challenge['unit']),
			));
		}

		// store milestone
		if ($submit && !sizeof($error))
		{
			$sql_data = array(
				'title'				=> $post_data['title'],
				'target'			=> $post_data['target'],
				'challenge_id'		=> $post_data['challenge_id'],
			);

			if ($mode == 'createmilestone')
			{
				$sql = 'INSERT INTO ' . MOD_CHALLENGE_MILESTONE_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_data);
				$db->sql_query($sql);
			}
			else if ($mode == 'editmilestone')
			{
				$sql = 'UPDATE ' . MOD_CHALLENGE_MILESTONE_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $sql_data) . '
					WHERE milestone_id = ' . $db->sql_escape($milestone['milestone_id']);
				$db->sql_query($sql);
			}
			unset($sql_data);

			// show message and redirect
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&amp;c=' . $post_data['challenge_id']);
			meta_refresh(3, $redirect_url);
			$message = $user->lang['MILESTONE_STORED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
			trigger_error($message);
		}

		// if editing and not yet submitted for preview, get milestone from db and prepare it for display
		if ($mode == 'editmilestone' && !$preview)
		{
			// $challenge and $milestone are set at the beginnnig of $mode == 'editmilestone'
			$post_data['title']				= $milestone['title'];
			$post_data['target']			= $milestone['target'];
		}

		$s_hidden_fields		 = '<input type="hidden" name="c" value="' . $challenge['challenge_id'] . '" />';
		if ($mode == 'editmilestone')
		{
			$s_hidden_fields	.= '<input type="hidden" name="m" value="' . $milestone['milestone_id'] . '" />';
		}

		add_form_key('creatingmilestone');

		// cut off the decimal point and the following zeroes for a nicer display
		$target = (intval($post_data['target']) == $post_data['target'] ? intval($post_data['target']) : $post_data['target']);
		$template->assign_vars(array(
			'ERROR'					=> (sizeof($error)) ? implode('<br />', $error) : '',
			'TITLE'					=> $post_data['title'],
			'TARGET'				=> ($target == 0 ? '' : $target),

			'S_POST_ACTION'			=> append_sid("{$phpbb_root_path}challenge.$phpEx", "mode=$mode"),
			'S_HIDDEN_FIELDS'		=> $s_hidden_fields,
		));

		$page_title = ($mode == 'createmilestone' ? $user->lang['TITLE_CREATE_MILESTONE'] : $user->lang['TITLE_EDIT_MILESTONE']);
		$template_html = 'challenge_create_milestone.html';
	}
	break;

	case 'deletemilestone':
	// delete a milestone
	{
		if ($milestone_id == 0)
		{
			trigger_error('NO_MILESTONE');
		}

		// we also need to get the "owner" of the challenge for permission check later
		$sql = 'SELECT m.*, c.user_id AS challenge_owner
			FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . ' m
			LEFT OUTER JOIN ' . MOD_CHALLENGE_CHALLENGE_TABLE . ' c ON m.challenge_id = c.challenge_id
			WHERE milestone_id = ' . $db->sql_escape(utf8_clean_string($milestone_id));
		$result = $db->sql_query($sql);
		$milestone = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$milestone)
		{
			trigger_error('NO_MILESTONE');
		}

		// check if logged in user is permitted to delete
		$deletepermission = ((MOD_CHALLENGE_PERM_U_M && (int) $user->data['user_id'] == (int) $milestone['challenge_owner'])
			|| (MOD_CHALLENGE_PERM_M_M && $auth->acl_get('m_'))
			|| (MOD_CHALLENGE_PERM_A_M && $auth->acl_get('a_'))
		);

		if (!$deletepermission)
		{
			trigger_error('NOT_AUTHORISED');
		}
		else
		{
			if (confirm_box(true))
			{
				// this code is executed if the user confirms to delete the milestone

				// delete the milestone
				$sql = 'DELETE FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
					WHERE ' . $db->sql_in_set('milestone_id', $milestone['milestone_id']);
				$db->sql_query_limit($sql, 1);

				$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&c=' . $challenge_id);
				meta_refresh(3, $redirect_url);

				$message = $user->lang['MILESTONE_DELETED'] . '<br /><br /><a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
				trigger_error($message);
			}
			else
			{
				// this code is executed first to show the confirmation dialog
				$s_hidden_fields = build_hidden_fields(array(
					'mode'		=> $mode,
					'm'			=> $milestone_id,
					'c'			=> $milestone['challenge_id']
				));

				confirm_box(false, 'DELETE_MILESTONE_P', $s_hidden_fields);
			}

			// this code is executed if the user declines to delete the log
			$redirect_url = append_sid("{$phpbb_root_path}challenge.$phpEx", 'mode=viewchallenge&c=' . $challenge_id);
			meta_refresh(3, $redirect_url);

			$message = '<a href="' . $redirect_url . '">' . $user->lang['VIEW_CHALLENGE'] . '</a>';
			trigger_error($message);
		}
	}
	break;
}

page_header($page_title);

$template->set_filenames(array(
	'body' => $template_html,
));

page_footer();


/**
* Parse date string (german format) into UNIX timestamp
*
* @param string	$date	The date to parse.
*
* @return int or null	Returns the parsed date as UNIX timestamp, or null in case of errors or invalid date string.
*/
function parse_date($date)
{
	$day	= substr($date, 0, 2);
	$month	= substr($date, 3, 2);
	$year	= substr($date, 6, 4);
	if ($day == false || $day == '' || !is_numeric($day) || $month == false || $month == '' || !is_numeric($month) || $year == false || $year == '' || !is_numeric($year))
	{
		return null;
	}
	$day = intval($day);
	$month = intval($month);
	$year = intval($year);
	if (!checkdate($month, $day, $year))
	{
		return null;
	}
	return mktime(0, 0, 0, $month, $day, $year);
}
?>