<?php
/**
*
* phpBB MOD Challenge graphics helper script
*
* @author Martin - http://vegan-forum.de/
*
* @package challenge
* @version 0.7
* @copyright (c) 2014 Martin
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

/* *** MOD settings ******************************************************** */
// the database tables of this MOD
define('MOD_CHALLENGE_CHALLENGE_TABLE',		$table_prefix . 'mod_challenge_challenge');
define('MOD_CHALLENGE_LOG_TABLE',			$table_prefix . 'mod_challenge_log');
define('MOD_CHALLENGE_MILESTONE_TABLE',		$table_prefix . 'mod_challenge_milestone');
define('MOD_CHALLENGE_PARTICIPANT_TABLE',	$table_prefix . 'mod_challenge_participant');
/* *** END MOD settings **************************************************** */


// grab data
$mode			= request_var('mode', '');
$challenge_id	= request_var('c', 0);
$parameter_p	= request_var('p', -1.0);

// check our mode...
if (!in_array($mode, array('', 'progressimg', 'simplebar')))
{
	trigger_error('NO_MODE');
}

switch ($mode)
{
	case 'progressimg':
	default:
	// display a progress image to a given challenge
	{
		$mode = 'progressimg';
		if ($challenge_id == 0)
		{
			trigger_error('NO_CHALLENGE');
		}

		$sql = 'SELECT *
			FROM ' . MOD_CHALLENGE_CHALLENGE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape(utf8_clean_string($challenge_id));
		$result = $db->sql_query($sql);
		$challenge = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$challenge)
		{
			trigger_error('NO_CHALLENGE');
		}

		// get information on logs to this challenge
		$sql = 'SELECT SUM(quantity) AS sum_quantity
			FROM ' . MOD_CHALLENGE_LOG_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']);
		$result = $db->sql_query($sql);
		$sum_quantity = $db->sql_fetchfield('sum_quantity');
		$db->sql_freeresult($result);
		$progress = min(100, max(0, floor($sum_quantity * 100 / $challenge['target'])));

		// get information for linked milestones
		$milestones = array();
		$sql = 'SELECT target
			FROM ' . MOD_CHALLENGE_MILESTONE_TABLE . '
			WHERE challenge_id = ' . $db->sql_escape($challenge['challenge_id']) .'
			ORDER BY target ASC';
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$milestones[] = min(100, max(0, floor($row['target'] * 100 / $challenge['target'])));
		}
		$db->sql_freeresult($result);

		output_gfx($progress, $milestones);
	}
	break;

	case 'simplebar':
	// display a progress bar without a challenge given, by the given parameter "p"
	// .../challenge_gfx.php?mode=simplebar&p=13.2
	{
		if ($parameter_p == -1.0)
		{
			trigger_error('NO_PROGRESS');
		}
		$progress = sprintf('%.2f', min(100, max(0, $parameter_p)));
		output_gfx($progress, null);
	}
	break;
}

function output_gfx($progress, $milestones) {
		// start output
		header('Content-type: image/svg+xml');
		echo '<svg xmlns="http://www.w3.org/2000/svg"
	xmlns:xlink="http://www.w3.org/1999/xlink"
	viewBox="0 0 101 11">
<defs>
	<linearGradient id="bar" x1="0%" y1="0%" x2="0%" y2="100%" spreadMethod="pad">
		<stop offset="0.2358" style="stop-color:#62CA0E" />
		<stop offset="0.4209" style="stop-color:#6CCE12" />
		<stop offset="0.5884" style="stop-color:#7DD51A" />
		<stop offset="0.6854" style="stop-color:#8ADB21" />
		<stop offset="0.9944" style="stop-color:#5FC90C" />
	</linearGradient>
	<linearGradient id="back" x1="0%" y1="0%" x2="0%" y2="100%" spreadMethod="pad">
		<stop offset="0" style="stop-color:#CFCECE" />
		<stop offset="0.2893" style="stop-color:#D2D1D1" />
		<stop offset="0.5163" style="stop-color:#DBDBDB" />
		<stop offset="0.6854" style="stop-color:#E8E8E8" />
		<stop offset="0.9944" style="stop-color:#CFCFCF" />
	</linearGradient>
	<linearGradient id="border" x1="0%" y1="0%" x2="100%" y2="0%" spreadMethod="pad">
		<stop offset="0" style="stop-color:#696969" />
		<stop offset="0.9944" style="stop-color:#A8A8A8" />
	</linearGradient>
</defs>
<rect x="0.5" y="0.5" width="100" height="10" rx="0" ry="0"
	style="fill: url(#back); stroke: none;" />
<rect x="0.5" y="0.5" width="'. $progress .'" height="10" rx="0" ry="0"
	style="fill: url(#bar); stroke: none;" />
<rect x="0.25" y="0.25" width="100.5" height="10.5" rx="1" ry="1"
	style="fill: none; stroke: url(#border); stroke-width: 0.5;" />';
		foreach ($milestones as $m)
		{
			echo '<line x1="'. $m .'" y1="0.5" x2="'. $m .'" y2="10.5" style="stroke:#007700; stroke-width: 0.5; fill: none;" />';
		}
		echo '<text x="50" y="8"
	style="text-anchor: middle; font-size: 6px;
	font-family: Arial;
	fill: #000000; stroke: none;
	">'. $progress .'%</text>
</svg>';
}
?>